/*-----------------------------------------------------------------------------*
// include
*-----------------------------------------------------------------------------*/
#include "harktool.h"

// extern
extern struct GLOBAL g;
extern bool hasParsedOpt;

void helpForCalcSysImp(poptContext con, int exitCode) {
    if (hasParsedOpt)
        return;
    printf("\n");
    printf("\n");
    printf("Function:\n");
    printf("  calcsysimp\n");
    printf("\n");
    printf("Description:\n");
    printf("  Calculate impluse responses from a recorded TSP wave\n");
    printf("\n");
    poptPrintHelp(con, stdout, 0);
    printf("\n");
    printf("Either of --tsporigflt or --tsporigwav should be given.\n");
    printf("--inputwave and --outputpath should be given.\n");
    printf("\n");
    printf("\n");
    exit(exitCode);
}

/*-----------------------------------------------------------------------------*
* calcSysImp
*-----------------------------------------------------------------------------*/
int harktoolcli_calcsysimp(int argc, char *argv[])
{
    enum SoundType orgType = -1;
    char* tspOrgFltFile = NULL;
    char* tspOrgWavFile = NULL;
    int tspOrgChannel = 0;
    int signalMax = 0;
    int samplesByFrame = 16384;
    int syncAddFreq = 16;
    int syncAddOffset = 16384;
    char* searchOffsetStr = NULL;
    long searchOffsetFrom = -1;
    long searchOffsetTo = -1;
    char* outChannelListStr = NULL;
    bool outChannels[MAX_CH];
    memset(outChannels, true, sizeof(bool) * MAX_CH);
    char* inputWaveFile = NULL;
    int chOrigin = 0;
    char* outputFltFile = NULL;
    char* logLevel = "E";
    struct poptOption optionsTable[] = {
    /*{ "longname", "shortname",
     *  argInfo,
     *  *arg, retval,
     *  description, argment description} */
    { "tsporigflt", '\0',
      POPT_ARG_STRING,
      &tspOrgFltFile, 'F',
      "Original tsp filename (float-data format)", NULL },
    { "tsporigwav", '\0',
      POPT_ARG_STRING,
      &tspOrgWavFile, 'W',
      "Original tsp filename (PCM-wave format)", NULL },
    { "tsporigchannel", '\0',
      POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &tspOrgChannel, 'r',
      "Channel number (for the PCM-wave tsp)", NULL },
    { "sigmax", '\0',
      POPT_ARG_INT,
      &signalMax, 'm',
      "Maximum signal value (for float-format tsp)", NULL },
    { "samplebyframe", '\0',
      POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &samplesByFrame, 'n',
      "Number of samplings per frame", NULL },
    { "syncaddfreq", '\0',
      POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &syncAddFreq, 'f',
      "Frequency of synchronous addition", NULL },
    { "syncaddoffset", '\0',
      POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &syncAddOffset, 'a',
      "Start position of synchronous addition", NULL },
    { "searchoffset", '\0',
      POPT_ARG_STRING,
      &searchOffsetStr, 's',
      "Start and end point of the search range. This enables to search a start position and use it.", "INT,INT" },
    { "outchannellist", '\0',
      POPT_ARG_STRING,
      &outChannelListStr, 'l',
      "Select channel(s) to calculate. (zero-based, comma separated)", "INT,..." }, // used no-break-space
    { "inputwave", '\0',
      POPT_ARG_STRING,
      &inputWaveFile, 'i',
      "Recorded wav file", NULL },
    { "outputpath", '\0',
      POPT_ARG_STRING,
      &outputFltFile, 'o',
      "Path prefix for output impulse response files. Output file will be 32bit float monaural raw data named '<prefix>_ch0.flt','<prefix>_ch1.flt',...", NULL },
    { "ch-origin", '\0',
      POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &chOrigin, 'g',
      "Number of first channel. This is used as the channel number of output file name.", NULL },
    { "logLevel", '\0',
      POPT_ARG_STRING | POPT_ARGFLAG_SHOW_DEFAULT,
      &logLevel, '-',
      "Log information level", "E|W|I|D" },
    { "help", '\0',
      0,
      NULL, '?',
      "Show this help message", NULL },
    POPT_TABLEEND };

    poptContext optCon = poptGetContext(NULL, argc, (const char**)argv, optionsTable, 0);
    poptSetOtherOptionHelp(optCon, "calcsysimp [OPTIONS]*");

    // option processing
    int c;
    while ((c = poptGetNextOpt(optCon)) >= 0) {
        switch (c) {
            char *tokenFrom, *tokenTo, *tokenCh, *end;
        case 'F':
            // if it is specify both of --tsporigflt,wav, the last argument specified to win.
            orgType = RAW;
            hasParsedOpt = true;
            break;
        case 'W':
            orgType = WAV;
            hasParsedOpt = true;
            break;
        case 'r': // tsporigchannel
            if (tspOrgChannel < 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 'm': // sigmax
            if (signalMax <= 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 'n': // samplebyframe
            if (samplesByFrame <= 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 'f': // syncaddfreq
            if (syncAddFreq <= 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 'a': // syncaddoffset
            if (syncAddOffset < 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 's': // searchoffset
            tokenFrom = strtok(searchOffsetStr, ",");
            tokenTo = strtok(NULL, ",");
            if (tokenFrom && tokenTo) {
                searchOffsetFrom = strtol(tokenFrom, &end, 10);
                if (!(end && *end == '\0') || searchOffsetFrom > INT_MAX || searchOffsetFrom < 0) {
                    c = POPT_ERROR_BADNUMBER;
                    goto OPTIONERR;
                }
                searchOffsetTo = strtol(tokenTo, &end, 10);
                if (!(end && *end == '\0') || searchOffsetTo > INT_MAX || searchOffsetTo < searchOffsetFrom) {
                    c = POPT_ERROR_BADNUMBER;
                    goto OPTIONERR;
                }
            } else {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case 'l': // outchannellist
            memset(outChannels, false, sizeof(bool) * MAX_CH);
            tokenCh = strtok(outChannelListStr, ", ");
            while (tokenCh != NULL) {
                long lCh = strtol(tokenCh, &end, 10);
                if (!(end && *end == '\0') || lCh > MAX_CH || lCh < 0) {
                    c = POPT_ERROR_BADNUMBER;
                    goto OPTIONERR;
                }
                if (!outChannels[lCh]) {
                    outChannels[lCh] = true;
                }
                tokenCh = strtok(NULL, ", ");
            }
            hasParsedOpt = true;
            break;
        case 'g': // ch-origin
            if (chOrigin < 0) {
                c = POPT_ERROR_BADNUMBER;
                goto OPTIONERR;
            }
            hasParsedOpt = true;
            break;
        case '-':
            if (strcmp(logLevel, "W") == 0) {
                harkio_Log_setLevel(harkio_Log_Warning);
            } else if (strcmp(logLevel, "I") == 0) {
                harkio_Log_setLevel(harkio_Log_Info);
            } else if (strcmp(logLevel, "D") == 0) {
                harkio_Log_setLevel(harkio_Log_Debug);
            } else {
                harkio_Log_setLevel(harkio_Log_Error);
            }
            hasParsedOpt = true;
            break;
        case '?':
            // help
            helpForCalcSysImp(optCon, EXIT_SUCCESS);
            /* no break */
        default:
            hasParsedOpt = true;
        }
    }
    if (c < -1) {
        OPTIONERR:
        /* an error occurred during option processing */
        harkio_Log_printf(harkio_Log_Error, "%s: %s",
                poptBadOption(optCon, POPT_BADOPTION_NOALIAS), poptStrerror(c));
        exit(EXIT_FAILURE);
    }

    // argment check
    bool argIsOK = true;
    if (orgType != RAW && orgType != WAV) {
        harkio_Log_printf(harkio_Log_Error, "missing --tsporigflt or --tsporigwav");
        argIsOK = false;
    }
    if (!inputWaveFile) {
        harkio_Log_printf(harkio_Log_Error, "missing --inputwave");
        argIsOK = false;
    }
    if (!outputFltFile) {
        harkio_Log_printf(harkio_Log_Error, "missing --outputpath");
        argIsOK = false;
    }
    if (!argIsOK) {
        exit(EXIT_FAILURE);
    }

    int ret = harktool_calcSysImp(orgType, (orgType==RAW)?tspOrgFltFile:tspOrgWavFile, tspOrgChannel,
            signalMax, 16000, samplesByFrame, syncAddFreq, syncAddOffset, searchOffsetFrom, searchOffsetTo,
            outChannels, chOrigin, inputWaveFile, outputFltFile);

    // get nmic
//    nmic = harktool_split(argv[outchannellist]  , "," ,  outlist) ;

//    harktool_mergeFltFile(argv[outputpath]               //char *in_file_path
//                          , argv[outimpfilepath]       // char *out_file_path
//                          , atoi(argv[posid])           // position id
//                          , outlist                     // channel list
//                          , nmic                        // num mic
//                          , atoi(argv[samplingfreq]));  // samplerate

    //void harktool_mergeFltFile( char *in_file_path , char *out_file_path , int *outlist  int nmic , int samplerate);

    /*--------------------------------------------------*/
    // end proc
    /*--------------------------------------------------*/
    harkio_Log_printf(harkio_Log_Info, "@ harktoolcli_calcsysimp %s end", (ret == EXIT_SUCCESS)?"normal":"error");

    // free poptContext
    poptFreeContext(optCon);

    harktool_Exit(ret);

    return EXIT_SUCCESS;
}

