#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <popt.h>
#include <math.h>
#include <limits.h>
#include <libharkio3.h>

bool hasParsedOpt = false;
void help(poptContext con, int exitCode) {
    if (hasParsedOpt)
        return;

    printf("\n");
    printf("\n");
    printf("NAME:\n");
    printf("  harktoolcli-conv-tf\n");
    printf("\n");
    printf("Description:\n");
    printf("  This command converts transfer function files in libharkio2-style\n");
    printf("  format to a new libharkio3-style format.\n");
    printf("  This command takes an old-style transfer function for localization\n");
    printf("  and/or one for separation. Then, it generastes a new-style file\n");
    printf("  format in zip files. If both transfer functions for localization\n");
    printf("  and separation are given, the command generates one zip file that\n");
    printf("  contains both two transfer functions.\n");
    printf("\n");
    printf("  If only a transfer function for localization is given, the\n");
    printf("  generated zip file has .mat files in its localization/ directory,\n");
    printf("  and the separation/ directory is empty. If both transfer functions\n");
    printf("  are given, both localization/ and separation/ directories have\n");
    printf("  corresponding .mat files.\n");
    printf("\n");
    poptPrintHelp(con, stdout, 0);
    printf("\n");
    printf("\n");
    exit(exitCode);
}

int main(int argc, char **argv)
{
    char *loctfpath = NULL;
    char *septfpath = NULL;
    char *outfile_ = NULL;
    char *whatisthis = "transfer function";
    char* logLevel = "E";

    harkio_TransferFunction *tf = NULL;

    struct poptOption optionsTable[] = {
    /*{ "longname", "shortname",
     *  argInfo,
     *  *arg, retval,
     *  description, argment description} */
    { "localization", 'l',
      POPT_ARG_STRING,
      &loctfpath, 'l',
      "Localization transfer file(Binary)", NULL },
    { "separation", 's',
      POPT_ARG_STRING,
      &septfpath, 's',
      "Separation transfer file(Binary)", NULL },
    // -o is for backward compatible with harktool4
    { "output", 'o',
      POPT_ARG_STRING | POPT_ARGFLAG_DOC_HIDDEN,
      &outfile_, 'o',
      "Output transfer file(zip)", NULL },
    { "logLevel", '\0',
      POPT_ARG_STRING | POPT_ARGFLAG_SHOW_DEFAULT,
      &logLevel, '-',
      "Log information level", "E|W|I|D" },
    { "help", '\0',
      0,
      NULL, '?',
      "Show this help message", NULL },
    POPT_TABLEEND };

    poptContext optCon = poptGetContext(NULL, argc, (const char**)argv, optionsTable, 0);
    poptSetOtherOptionHelp(optCon, "[OPTIONS]* outputfile(zip)");

    //argument check
    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help(optCon, EXIT_FAILURE);
    }

    // option processing
    int c;
    while ((c = poptGetNextOpt(optCon)) >= 0) {
        switch (c) {
        case '-':
            if (strcmp(logLevel, "W") == 0) {
                harkio_Log_setLevel(harkio_Log_Warning);
            } else if (strcmp(logLevel, "I") == 0) {
                harkio_Log_setLevel(harkio_Log_Info);
            } else if (strcmp(logLevel, "D") == 0) {
                harkio_Log_setLevel(harkio_Log_Debug);
            } else {
                harkio_Log_setLevel(harkio_Log_Error);
            }
            hasParsedOpt = true;
            break;
        case '?':
            // help
            help(optCon, EXIT_SUCCESS);
            /* no break */
        default:
            hasParsedOpt = true;
        }
    }
    if (c < -1) {
        /* an error occurred during option processing */
        harkio_Log_printf(harkio_Log_Error, "%s: %s",
                poptBadOption(optCon, POPT_BADOPTION_NOALIAS), poptStrerror(c));
        exit(EXIT_FAILURE);
    }

    if (loctfpath) printf("loctfpath=%s\n",loctfpath);
    if (septfpath) printf("septfpath=%s\n",septfpath);

    if (!loctfpath && !septfpath) {
        harkio_Log_printf(harkio_Log_Error , "one of -l or -s is needed at least");
        exit(EXIT_FAILURE);
    }
    // result = libharkio3_TransferFunction_readlibhakio2( &tf , loctfpath ,  septfpath);
    tf = harkio_Compat_TransferFunction_readlibharkio2(loctfpath, septfpath);

    if (tf == NULL) {
        harkio_TransferFunction_delete(&tf);
        harkio_Log_printf(harkio_Log_Error , "harkio_Compat_TransferFunction_readlibharkio2 error");
        exit(EXIT_FAILURE);
    }

    // get argment
    const char* outfile = poptGetArg(optCon);

    if(outfile_ && outfile) {
        harkio_Log_printf(harkio_Log_Error, "%s: %s",
                poptBadOption(optCon, POPT_BADOPTION_NOALIAS), poptStrerror(POPT_ERROR_BADOPERATION));
        exit(EXIT_FAILURE);
    }

    if (outfile_ && !outfile)
        outfile = outfile_;

    if (!outfile) {
        harkio_Log_printf(harkio_Log_Error , "missing outputfile");
        exit(EXIT_FAILURE);
    }

    //write zip file
    harkio_TransferFunction_write(tf , outfile , whatisthis);

    fprintf(stderr, "SUCCESS\n");
    fprintf(stderr, " output file : %s\n",outfile);

    harkio_TransferFunction_delete(&tf);

    // free poptContext
    poptFreeContext(optCon);

    exit(EXIT_SUCCESS);
}//func end


